#include "Menu.h"
#include <Configuration.h>
#include <States/ChangeStateCommand.h>
#include <Themes/ChangeThemeCommand.h>
#include <Themes/Themes.h>

Menu::Menu(IStateManager& state_manager, SelectState& select_state) {
    const float pos_left = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.width) - config::BUTTON_SIZE.x) / 2;
    const float pos_diff = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.height) - config::BUTTON_SIZE.y * 5 - config::BUTTON_CHANGE_THEME_SIZE.y) / 10;
    const float pos_top = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.height) - config::BUTTON_SIZE.y*5 - pos_diff*5 - config::BUTTON_CHANGE_THEME_SIZE.y) / 2;

    const auto width = config::GAME_VIDEO_MODE.width;
    const auto height = config::GAME_VIDEO_MODE.height;
    const auto room_color = Themes::Instance().get_game_color_room();

    auto easy_level_command = std::make_unique<GameCommand>(
        state_manager,
        std::make_unique<GameBuilderDirector>(
            std::make_unique<SimpleBuilder>(width, height, config::ROOM_SIZE, room_color, 0),
            config::GAME_VIDEO_MODE,
            config::EASY_GAME_TITLE,
            config::EASY_GAME_ENEMY_RATIO));

    auto medium_level_command = std::make_unique<GameCommand>(
        state_manager,
        std::make_unique<GameBuilderDirector>(
            std::make_unique<SimpleBuilder>(width, height, config::ROOM_SIZE, room_color, config::ENEMY_SPEED_MEDIEUM),
            config::GAME_VIDEO_MODE,
            config::MEDIUM_GAME_TITLE,
            config::MEDIUM_GAME_ENEMY_RATIO));

    auto hard_level_command = std::make_unique<GameCommand>(
        state_manager,
        std::make_unique<GameBuilderDirector>(
            std::make_unique<ComplexBuilder>(width, height, config::ROOM_SIZE, room_color, config::ENEMY_SPEED_HARD),
            config::GAME_VIDEO_MODE,
            config::HARD_GAME_TITLE,
            config::HARD_GAME_ENEMY_RATIO));

    auto extreme_level_command = std::make_unique<GameCommand>(
        state_manager,
        std::make_unique<GameBuilderDirector>(
            std::make_unique<ComplexRandomBuilder>(width, height, config::ROOM_SIZE, room_color, config::ENEMY_SPEED_EXTREME),
            config::GAME_VIDEO_MODE,
            config::EXTREME_GAME_TITLE,
            config::HARD_GAME_ENEMY_RATIO));

    auto change_theme_command = std::make_unique<ChangeThemeCommand>(select_state);

    m_buttons[0].set(sf::Vector2f{pos_left, pos_top}, config::BUTTON_SIZE, config::BUTTON_TEXT_EASY,
        config::BUTTON_FONT_SIZE, std::move(easy_level_command));
    m_buttons[1].set(sf::Vector2f{pos_left, pos_top + config::BUTTON_SIZE.y + pos_diff}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_MEDIUM, config::BUTTON_FONT_SIZE, std::move(medium_level_command));
    m_buttons[2].set(sf::Vector2f{pos_left, pos_top + 2*(config::BUTTON_SIZE.y + pos_diff)}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_HARD, config::BUTTON_FONT_SIZE, std::move(hard_level_command));
    m_buttons[3].set(sf::Vector2f{pos_left, pos_top + 3*(config::BUTTON_SIZE.y + pos_diff)}, config::BUTTON_SIZE,
    config::BUTTON_TEXT_EXTREME, config::BUTTON_FONT_SIZE, std::move(extreme_level_command));
    m_buttons[4].set(sf::Vector2f{pos_left, pos_top + 4*(config::BUTTON_SIZE.y + pos_diff)}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_EXIT, config::BUTTON_FONT_SIZE, std::make_unique<ExitCommand>(state_manager));
    m_buttons[5].set(sf::Vector2f{pos_left + config::BUTTON_SIZE.x/3.f,
        pos_top + 5*(config::BUTTON_SIZE.y + pos_diff) + config::BUTTON_CHANGE_THEME_SIZE.y/3.f}, config::BUTTON_CHANGE_THEME_SIZE,
            Themes::Instance().get_button_change_theme_text(), config::BUTTON_CHANGE_THEME_FONT_SIZE,
     std::move(change_theme_command));
}

void Menu::process_mouse(const sf::Vector2f pos, const bool is_pressed) {
    for (auto& button : m_buttons) {
        if (!button.is_position_in(pos)) {
            button.unselect();
            continue;
        }
        if (!is_pressed) {
            button.select();
            continue;
        }
        button.push();
    }
}

void Menu::draw_into(sf::RenderWindow& window) const {
    for (auto& button : m_buttons) {
        button.draw_into(window);
    }
}